require "git"
require "tty-spinner"
require "active_support"
require "active_support/core_ext"

require "app"
require "api_url"
require "api_token"
require "aws_folder_upload"

module Flubber
  class Deploy
    FILE_ALL_EXPO_APPS_HAVE = ".expo"
    FOLDERS_TO_UPLOAD = [{
      name: "dist",
      aws_destination: "/",
    }, {
      name: "dist/assets",
      aws_destination: "assets",
    }, {
      name: "dist/bundles",
      aws_destination: "bundles",
    }]

    def self.deploy
      new.deploy
    end

    def deploy
      unless working_directory_is_an_expo_project?
        puts "Failed. Run me in an expo project."
        return
      end
      remove_bundle
      set_flubber_app
      puts "Using app #{App.name}"
      generate_bundle
      upload_bundle
      remove_bundle
      log_deploy
    end

    private

    attr_reader :selection

    def working_directory_is_an_expo_project?
      @working_directory_is_an_expo_project ||= \
        File.exist?(FILE_ALL_EXPO_APPS_HAVE)
    end

    def set_flubber_app
      return if flubber_app_is_set?
      ask_for_which_flubber_app_to_use
    end

    def flubber_app_is_set?
      App.set?
    end

    def flubber_apps
      @flubber_apps ||= JSON.parse(Faraday.get(
        "#{API_URL}/apps",
        {},
        { "Authorization" => "Bearer #{api_token}" },
      ).body)
    end

    def print_existing_app_options
      flubber_apps.each_with_index do |app, index|
        puts "#{index + 1}. #{app["name"]}"
      end
    end

    def print_new_app_option
      puts "#{flubber_apps.count + 1}. Create A New Flubber App"
    end

    def existing_app_selected?
      selection >= 1 && selection <= flubber_apps.count
    end

    def new_app_selected?
      selection == flubber_apps.count + 1
    end

    def use_existing_app_selection
      flubber_app_index = selection - 1
      App.set(flubber_apps[flubber_app_index])
    end

    def create_new_app
      print "Name your app: "
      name = STDIN.gets.chomp
      app = JSON.parse(Faraday.post(
        "#{API_URL}/apps",
        { "name": name },
        { "Authorization" => "Bearer #{api_token}" },
      ).body)
      App.set(app)
    end

    def ask_for_which_flubber_app_to_use
      print_existing_app_options
      print_new_app_option
      loop do
        print "Which app? "
        @selection = STDIN.gets.chomp.to_i
        if existing_app_selected?
          use_existing_app_selection
          break
        elsif new_app_selected?
          create_new_app
          break
        end
        puts "Not an option"
      end
    end

    def generate_bundle
      bundling_spinner = TTY::Spinner.new(
        "[:spinner] Bundling ...",
        format: :dots,
      )
      bundling_spinner.auto_spin
      `expo export --public-url #{App.bundles_public_url}`
      bundling_spinner.stop('Done!')
    end

    def upload_bundle
      uploading_spinner = TTY::Spinner.new(
        "[:spinner] Uploading ...",
        format: :dots,
      )
      uploading_spinner.auto_spin
      FOLDERS_TO_UPLOAD.each do |folder|
        AwsFolderUpload.upload(
          folder_name: folder[:name],
          aws_destination: folder[:aws_destination],
        )
      end
      uploading_spinner.stop('Done!')
    end

    def log_deploy
      Faraday.post(
        "#{API_URL}/apps/#{App.id}/updates",
        { "commit_sha": sha },
        { "Authorization" => "Bearer #{api_token}" },
      )
    end

    def sha
      @sha ||= Git.open(".").log.first.sha
    end

    def api_token
      @api_token ||= ApiToken.get
    end

    def remove_bundle
      `rm -rf dist`
    end
  end
end
